X-Git-Url: https://git.saurik.com/wxWidgets.git/blobdiff_plain/fd3f686c274a264e89ea97505350a82c1134f307..f42b1601d6a55f3c2b7e7c22894727bda70d8505:/include/wx/dynarray.h
diff --git a/include/wx/dynarray.h b/include/wx/dynarray.h
index 6fd78a428f..caf4b814d1 100644
--- a/include/wx/dynarray.h
+++ b/include/wx/dynarray.h
@@ -19,7 +19,7 @@
#include "wx/defs.h"
#include "wx/debug.h"
-/** @name Dynamic arrays and lists
+/** @name Dynamic arrays and object arrays (array which own their elements)
@memo Arrays which grow on demand and do range checking (only in debug)
*/
//@{
@@ -29,7 +29,7 @@
// ----------------------------------------------------------------------------
/**
- the initial size by which an array/list grows when an element is added
+ the initial size by which an array grows when an element is added
default value avoids allocate one or two bytes when the array is created
which is rather inefficient
*/
@@ -44,8 +44,10 @@
must return negative value, 0 or positive value if pItem1 <, = or > pItem2
*/
-#ifdef __VISUALC__
+#if defined(__VISUALC__)
#define CMPFUNC_CONV _cdecl
+#elif defined(__VISAGECPP__)
+ #define CMPFUNC_CONV _Optlink
#else // !Visual C++
#define CMPFUNC_CONV
#endif // compiler
@@ -61,7 +63,7 @@ typedef int (CMPFUNC_CONV *CMPFUNC)(const void* pItem1, const void* pItem2);
so it's not too important if it's not called (this happens when you cast
"SomeArray *" as "BaseArray *" and then delete it)
- @memo Base class for template array and list classes
+ @memo Base class for template array classes
*/
// ----------------------------------------------------------------------------
class WXDLLEXPORT wxBaseArray
@@ -85,9 +87,9 @@ public:
/** @name memory management */
//@{
- /// empties the list, but doesn't release memory
+ /// empties the array, but doesn't release memory
void Empty() { m_nCount = 0; }
- /// empties the list and releases memory
+ /// empties the array and releases memory
void Clear();
/// preallocates memory for given number of items
void Alloc(size_t uiSize);
@@ -106,7 +108,7 @@ public:
protected:
// these methods are protected because if they were public one could
// mistakenly call one of them instead of DEFINE_ARRAY's or LIST's
// type safe methods
/** @name items access */
@@ -123,8 +125,8 @@ protected:
/**
Search the element in the array, starting from the either side
@param bFromEnd if TRUE, start from the end
- @return index of the first item matched or NOT_FOUND
- @see NOT_FOUND
+ @return index of the first item matched or wxNOT_FOUND
+ @see wxNOT_FOUND
*/
int Index(long lItem, bool bFromEnd = FALSE) const;
/// search for an item using binary search in a sorted array
@@ -162,17 +164,26 @@ private:
// types of sizeof()<=sizeof(long) or pointers if sizeof(pointer)<=sizeof(long)
//
// NB: it has only inline functions => takes no space at all
+// Mod by JACS: Salford C++ doesn't like 'var->operator=' syntax, as in:
+// { ((wxBaseArray *)this)->operator=((const wxBaseArray&)src);
+// so using a temporary variable instead.
// ----------------------------------------------------------------------------
#define _WX_DEFINE_ARRAY(T, name) \
typedef int (CMPFUNC_CONV *CMPFUNC##T)(T *pItem1, T *pItem2); \
-class WXDLLEXPORTLOCAL name : public wxBaseArray \
+class WXDLLEXPORT name : public wxBaseArray \
{ \
public: \
name() \
- { wxASSERT( sizeof(T) <= sizeof(long) ); } \
+ { \
+ size_t type = sizeof(T); \
+ size_t sizelong = sizeof(long); \
+ if ( type > sizelong ) \
+ { wxFAIL_MSG( _T("illegal use of DEFINE_ARRAY") ); } \
+ } \
\
name& operator=(const name& src) \
- { ((wxBaseArray *)this)->operator=((const wxBaseArray&)src); \
+ { wxBaseArray* temp = (wxBaseArray*) this; \
+ (*temp) = ((const wxBaseArray&)src); \
return *this; } \
\
T& operator[](size_t uiIndex) const \
@@ -193,9 +204,9 @@ public: \
void Remove(size_t uiIndex) { wxBaseArray::Remove(uiIndex); } \
void Remove(T Item) \
{ int iIndex = Index(Item); \
- wxCHECK2_MSG( iIndex != NOT_FOUND, return, \
- "removing inexisting element in wxArray::Remove" ); \
- wxBaseArray::Remove((size_t)iIndex); } \
+ wxCHECK2_MSG( iIndex != wxNOT_FOUND, return, \
+ _T("removing inexisting element in wxArray::Remove") ); \
+ wxBaseArray::Remove((size_t)iIndex); } \
\
void Sort(CMPFUNC##T fCmp) { wxBaseArray::Sort((CMPFUNC)fCmp); } \
}
@@ -216,17 +227,26 @@ public: \
// the normal arrays otherwise.
//
// NB: it has only inline functions => takes no space at all
+// Mod by JACS: Salford C++ doesn't like 'var->operator=' syntax, as in:
+// { ((wxBaseArray *)this)->operator=((const wxBaseArray&)src);
+// so using a temporary variable instead.
// ----------------------------------------------------------------------------
#define _WX_DEFINE_SORTED_ARRAY(T, name) \
typedef int (CMPFUNC_CONV *SCMPFUNC##T)(T pItem1, T pItem2); \
-class WXDLLEXPORTLOCAL name : public wxBaseArray \
+class WXDLLEXPORT name : public wxBaseArray \
{ \
public: \
name(SCMPFUNC##T fn) \
- { wxASSERT( sizeof(T) <= sizeof(long) ); m_fnCompare = fn; } \
+ { size_t type = sizeof(T); \
+ size_t sizelong = sizeof(long); \
+ if ( type > sizelong ) \
+ { wxFAIL_MSG( _T("illegal use of DEFINE_ARRAY") ); } \
+ m_fnCompare = fn; \
+ } \
\
name& operator=(const name& src) \
- { ((wxBaseArray *)this)->operator=((const wxBaseArray&)src); \
+ { wxBaseArray* temp = (wxBaseArray*) this; \
+ (*temp) = ((const wxBaseArray&)src); \
m_fnCompare = src.m_fnCompare; \
return *this; } \
\
@@ -246,8 +266,8 @@ public: \
void Remove(size_t uiIndex) { wxBaseArray::Remove(uiIndex); } \
void Remove(T Item) \
{ int iIndex = Index(Item); \
- wxCHECK2_MSG( iIndex != NOT_FOUND, return, \
- "removing inexisting element in wxArray::Remove" ); \
+ wxCHECK2_MSG( iIndex != wxNOT_FOUND, return, \
+ _T("removing inexisting element in wxArray::Remove") ); \
wxBaseArray::Remove((size_t)iIndex); } \
\
private: \
@@ -255,11 +275,11 @@ private: \
}
// ----------------------------------------------------------------------------
-// see WX_DECLARE_LIST and WX_DEFINE_LIST
+// see WX_DECLARE_OBJARRAY and WX_DEFINE_OBJARRAY
// ----------------------------------------------------------------------------
-#define _WX_DECLARE_LIST(T, name) \
+#define _WX_DECLARE_OBJARRAY(T, name) \
typedef int (CMPFUNC_CONV *CMPFUNC##T)(T** pItem1, T** pItem2); \
-class WXDLLEXPORTLOCAL name : public wxBaseArray \
+class WXDLLEXPORT name : public wxBaseArray \
{ \
public: \
name() { } \
@@ -299,17 +319,17 @@ private: \
}
// ----------------------------------------------------------------------------
/** @name Macros for definition of dynamic arrays and lists
These macros are ugly (especially if you look in the sources ;-), but they
allow us to define 'template' classes without actually using templates.
- Range checking is performed in debug build for both arrays and lists. Type
- checking is done at compile-time. Warning: arrays never shrink, they
- only grow, so loading 10 millions in an array only to delete them 2 lines
- below is not recommended. However, it does free memory when it's
- destroyed, so if you destroy array also, it's ok.
+ Range checking is performed in debug build for both arrays and objarrays.
+ Type checking is done at compile-time. Warning: arrays never shrink,
+ they only grow, so loading 10 millions in an array only to delete them 2
+ lines below is not recommended. However, it does free memory when
+ it's destroyed, so if you destroy array also, it's ok.
*/
// ----------------------------------------------------------------------------
@@ -334,60 +354,60 @@ private: \
_WX_DEFINE_SORTED_ARRAY(_A##name, name)
/**
- This macro generates a new list class which owns the objects it contains,
- i.e. it will delete them when it is destroyed. An element is of type T*,
- but arguments of type T& are taken (see below!) and T& is returned.
-
+ This macro generates a new objarrays class which owns the objects it
+ contains, i.e. it will delete them when it is destroyed. An element is of
+ type T*, but arguments of type T& are taken (see below!) and T& is
+ returned.
Don't use this for simple types such as "int" or "long"!
You _may_ use it for "double" but it's awfully inefficient.
Note on Add/Insert functions:
- 1) function(T*) gives the object to the list, i.e. it will delete the
- object when it's removed or in the list's dtor
+ 1) function(T*) gives the object to the array, i.e. it will delete the
+ object when it's removed or in the array's dtor
2) function(T&) will create a copy of the object and work with it
Also:
- 1) Remove() will delete the object after removing it from the list
+ 1) Remove() will delete the object after removing it from the array
- 2) Detach() just removes the object from the list (returning pointer to it)
+ 2) Detach() just removes the object from the array (returning pointer to it)
NB1: Base type T should have an accessible copy ctor if Add(T&) is used,
- NB2: Never ever cast a list to it's base type: as dtor is not virtual
+ NB2: Never ever cast a array to it's base type: as dtor is not virtual
it will provoke memory leaks
some functions of this class are not inline, so it takes some space to
define new class from this template.
- @memo declare list class 'name' containing elements of type 'T'
+ @memo declare objarray class 'name' containing elements of type 'T'
*/
#define WX_DECLARE_OBJARRAY(T, name) typedef T _L##name; \
- _WX_DECLARE_LIST(_L##name, name)
+ _WX_DECLARE_OBJARRAY(_L##name, name)
/**
- To use a list class you must
+ To use an objarray class you must
#include "dynarray.h"
- DECLARE_LIST(element_type, list_class_name)
- #include "listimpl.cpp"
- DEFINE_LIST(list_class_name) // same as above!
+ WX_DECLARE_OBJARRAY(element_type, list_class_name)
+ #include "arrimpl.cpp"
+ WX_DEFINE_OBJARRAY(list_class_name) // same as above!
- This is necessary because at the moment of DEFINE_LIST class element_type
- must be fully defined (i.e. forward declaration is not enough), while
- DECLARE_LIST may be done anywhere. The separation of two allows to break
- cicrcular dependencies with classes which have member variables of list
- type.
+ This is necessary because at the moment of DEFINE_OBJARRAY class
+ element_type must be fully defined (i.e. forward declaration is not
+ enough), while WX_DECLARE_OBJARRAY may be done anywhere. The separation of
+ two allows to break cicrcular dependencies with classes which have member
+ variables of objarray type.
- @memo define (must include listimpl.cpp!) list class 'name'
+ @memo define (must include arrimpl.cpp!) objarray class 'name'
*/
-#define WX_DEFINE_OBJARRAY(name) "don't forget to include listimpl.cpp!"
+#define WX_DEFINE_OBJARRAY(name) "don't forget to include arrimpl.cpp!"
//@}
// ----------------------------------------------------------------------------
@@ -395,8 +415,6 @@ private: \
// # overhead if not used?
// ----------------------------------------------------------------------------
-#define WXDLLEXPORTLOCAL WXDLLEXPORT
-
//@{
/** @name ArrayInt */
WX_DEFINE_ARRAY(int, wxArrayInt);
@@ -408,8 +426,25 @@ WX_DEFINE_ARRAY(void *, wxArrayPtrVoid);
//@}
-#undef WXDLLEXPORTLOCAL
-#define WXDLLEXPORTLOCAL
+// -----------------------------------------------------------------------------
+// convinience macros
+// -----------------------------------------------------------------------------
+// delete all array elements
+//
+// NB: the class declaration of the array elements must be visible from the
+// place where you use this macro, otherwise the proper destructor may not
+// be called (a decent compiler should give a warning about it, but don't
+// count on it)!
+#define WX_CLEAR_ARRAY(array) \
+ { \
+ size_t count = array.Count(); \
+ for ( size_t n = 0; n < count; n++ ) \
+ { \
+ delete array[n]; \
+ } \
+ \
+ array.Empty(); \
+ }
#endif // _DYNARRAY_H
@@ -106,7 +108,7 @@ public:
protected:
// these methods are protected because if they were public one could
- // mistakenly call one of them instead of DEFINE_ARRAY's or LIST's
+ // mistakenly call one of them instead of DEFINE_ARRAY's or OBJARRAY's
// type safe methods
/** @name items access */
@@ -299,17 +319,17 @@ private: \
}
// ----------------------------------------------------------------------------
-/** @name Macros for definition of dynamic arrays and lists
+/** @name Macros for definition of dynamic arrays and objarrays
These macros are ugly (especially if you look in the sources ;-), but they
allow us to define 'template' classes without actually using templates.